@@ -41,7 +41,7 @@ const gchar * glade_catalog_get_name                (GladeCatalog     *catalog);
 LIBGLADEUI_API 
 GList *       glade_catalog_get_widget_groups       (GladeCatalog     *catalog);
 LIBGLADEUI_API 
-GList *       glade_catalog_get_widget_classes      (GladeCatalog     *catalog);
+GList *       glade_catalog_get_adaptors            (GladeCatalog     *catalog);
 
 LIBGLADEUI_API 
 void          glade_catalog_free                    (GladeCatalog     *catalog);
@@ -53,7 +53,7 @@ const gchar * glade_widget_group_get_title          (GladeWidgetGroup *group);
 LIBGLADEUI_API 
 gboolean      glade_widget_group_get_expanded       (GladeWidgetGroup *group);
 LIBGLADEUI_API 
-GList *       glade_widget_group_get_widget_classes (GladeWidgetGroup *group);
+GList *       glade_widget_group_get_adaptors       (GladeWidgetGroup *group);
 
 LIBGLADEUI_API 
 void          glade_widget_group_free               (GladeWidgetGroup *group);
