@@ -276,15 +276,15 @@ glade_placeholder_expose (GtkWidget *widget, GdkEventExpose *event)
 static gboolean                                                                 
 glade_placeholder_motion_notify_event (GtkWidget *widget, GdkEventMotion *event)
 {
-	GladeWidgetClass *item_class;
-	GladeWidget      *gparent;
+	GladeWidgetAdaptor *adaptor;
+	GladeWidget        *gparent;
 
 	g_return_val_if_fail (GLADE_IS_PLACEHOLDER (widget), FALSE);
 
-	gparent   = glade_placeholder_get_parent (GLADE_PLACEHOLDER (widget));
-	item_class = glade_palette_get_current_item_class (glade_app_get_palette ());
+	gparent = glade_placeholder_get_parent (GLADE_PLACEHOLDER (widget));
+	adaptor = glade_palette_get_current_item (glade_app_get_palette ());
 
-	if (item_class == NULL && 
+	if (adaptor == NULL && 
 	    /* If we are the child of a widget that is in a GladeFixed, then
 	     * we are the means of drag/resize and we dont want to fight for
 	     * the cursor (ideally; GladeCursor should somehow deal with such
@@ -293,7 +293,7 @@ glade_placeholder_motion_notify_event (GtkWidget *widget, GdkEventMotion *event)
 	    (gparent->parent && 
 	     GLADE_IS_FIXED (gparent->parent)) == FALSE)
                 glade_cursor_set (event->window, GLADE_CURSOR_SELECTOR);
-	else if (item_class)
+	else if (adaptor)
                 glade_cursor_set (event->window, GLADE_CURSOR_ADD_WIDGET);
 
 	return FALSE;
@@ -302,15 +302,15 @@ glade_placeholder_motion_notify_event (GtkWidget *widget, GdkEventMotion *event)
 static gboolean
 glade_placeholder_button_press (GtkWidget *widget, GdkEventButton *event)
 {
-	GladePlaceholder *placeholder;
-	GladeProject     *project;
-	GladeWidgetClass *item_class;
-	GladePalette     *palette;
-	gboolean          handled = FALSE;
+	GladePlaceholder   *placeholder;
+	GladeProject       *project;
+	GladeWidgetAdaptor *adaptor;
+	GladePalette       *palette;
+	gboolean            handled = FALSE;
 
 	g_return_val_if_fail (GLADE_IS_PLACEHOLDER (widget), FALSE);
 
-	item_class = glade_palette_get_current_item_class (glade_app_get_palette ());
+	adaptor = glade_palette_get_current_item (glade_app_get_palette ());
 
 	palette = glade_app_get_palette ();
 	placeholder = GLADE_PLACEHOLDER (widget);
@@ -321,13 +321,13 @@ glade_placeholder_button_press (GtkWidget *widget, GdkEventButton *event)
 
 	if (event->button == 1 && event->type == GDK_BUTTON_PRESS)
 	{
-		if (item_class != NULL)
+		if (adaptor != NULL)
 		{
 			/* A widget type is selected in the palette.
 			 * Add a new widget of that type.
 			 */
 			glade_command_create
-				(item_class, 
+				(adaptor, 
 				 glade_placeholder_get_parent (placeholder),
 				 placeholder, project);
 
