@@ -31,18 +31,13 @@
 #include <glib/gstdio.h>
 
 #include "glade.h"
-#include "glade-project.h"
 #include "glade-widget.h"
-#include "glade-widget-class.h"
-#include "glade-xml-utils.h"
-#include "glade-widget.h"
-#include "glade-placeholder.h"
-#include "glade-editor.h"
-#include "glade-utils.h"
 #include "glade-id-allocator.h"
 #include "glade-app.h"
-#include "glade-catalog.h"
 #include "glade-marshallers.h"
+#include "glade-catalog.h"
+
+#include "glade-project.h"
 
 enum
 {
@@ -116,9 +111,9 @@ glade_project_dispose (GObject *object)
 		gwidget = glade_widget_get_from_gobject (list->data);
 
 		if (gwidget->parent &&
-		    glade_widget_class_container_has_child (gwidget->parent->widget_class,
-							    gwidget->parent->object,
-							    gwidget->object))
+		    glade_widget_adaptor_has_child (gwidget->parent->adaptor,
+						    gwidget->parent->object,
+						    gwidget->object))
 			glade_widget_remove_child (gwidget->parent, gwidget);
 	}
 
@@ -145,6 +140,7 @@ glade_project_finalize (GObject *object)
 
 	g_free (project->name);
 	g_free (project->path);
+	g_free (project->comment);
 
 	if (project->untitled_number > 0)
 		glade_project_release_untitled_number (project->untitled_number);
@@ -791,8 +787,8 @@ glade_project_sync_resources_for_widget (GladeProject *project,
 	GList *children, *l;
 	GladeWidget *gchild;
 
-	children = glade_widget_class_container_get_children
-		(gwidget->widget_class, gwidget->object);
+	children = glade_widget_adaptor_get_children
+		(gwidget->adaptor, gwidget->object);
 
 	for (l = children; l; l = l->next)
 		if ((gchild = 
@@ -841,8 +837,8 @@ glade_project_add_object (GladeProject *project,
 	/* Code body starts here */
 	reentrancy_count++;
 
-	if ((children = glade_widget_class_container_get_children
-	     (gwidget->widget_class, gwidget->object)) != NULL)
+	if ((children = glade_widget_adaptor_get_children
+	     (gwidget->adaptor, gwidget->object)) != NULL)
 	{
 		for (list = children; list && list->data; list = list->next)
 			glade_project_add_object
@@ -977,8 +973,9 @@ glade_project_remove_object (GladeProject *project, GObject *object)
 	/* Notify widget is being removed from the project */
 	glade_widget_project_notify (gwidget, NULL);
 	
-	if ((children = glade_widget_class_container_get_children (gwidget->widget_class,
-								   gwidget->object)) != NULL)
+	if ((children = 
+	     glade_widget_adaptor_get_children (gwidget->adaptor,
+						gwidget->object)) != NULL)
 	{
 		for (list = children; list && list->data; list = list->next)
 			glade_project_remove_object (project, G_OBJECT (list->data));
@@ -1303,28 +1300,80 @@ glade_project_required_libs (GladeProject *project)
 
 	for (l = project->objects; l; l = l->next)
 	{
+		gchar *catalog = NULL;
+
 		gwidget = glade_widget_get_from_gobject (l->data);
 		g_assert (gwidget);
 
-		if (gwidget->widget_class->catalog)
+		g_object_get (gwidget->adaptor, "catalog", &catalog, NULL);
+
+		if (catalog)
 		{
 			listed = FALSE;
 			for (ll = required; ll; ll = ll->next)
-				if (!strcmp ((gchar *)ll->data, 
-					     gwidget->widget_class->catalog))
+				if (!strcmp ((gchar *)ll->data, catalog))
 				{
 					listed = TRUE;
 					break;
 				}
 
 			if (!listed)
-				required = g_list_prepend 
-					(required, gwidget->widget_class->catalog);
+				required = g_list_prepend (required, catalog);
 		}
 	}
 	return required;
 }
 
+#define GLADE_XML_COMMENT "Generated with "PACKAGE_NAME
+
+static gchar *
+glade_project_make_comment ()
+{
+	time_t now = time (NULL);
+	gchar *comment;
+	comment = g_strdup_printf (GLADE_XML_COMMENT" "PACKAGE_VERSION" on %sby %s@%s",
+				   ctime (&now), g_get_user_name (), g_get_host_name ());
+	glade_util_replace (comment, '\n', ' ');
+	
+	return comment;
+}
+
+static void
+glade_project_update_comment (GladeProject *project)
+{
+	gchar **lines, **l, *comment = NULL;
+	
+	/* If project has no comment -> add the new one */
+	if (project->comment == NULL)
+	{
+		project->comment = glade_project_make_comment ();
+		return;
+	}
+	
+	for (lines = l = g_strsplit (project->comment, "\n", 0); *l; l++)
+	{
+		gchar *start;
+		
+		/* Ignore leading spaces */
+		for (start = *l; *start && g_ascii_isspace (*start); start++);
+		
+		if (g_str_has_prefix (start, GLADE_XML_COMMENT))
+		{
+			/* This line was generated by glade -> updating... */
+			g_free (*l);
+			*l = comment = glade_project_make_comment ();
+		}
+	}
+	
+	if (comment)
+	{
+		g_free (project->comment);
+		project->comment = g_strjoinv ("\n", lines);
+	}
+	
+	g_strfreev (lines);
+}
+
 /**
  * glade_project_write:
  * @project: a #GladeProject
@@ -1340,13 +1389,13 @@ glade_project_write (GladeProject *project)
 	gchar          **strv = NULL;
 	guint            i;
 
-	interface = glade_interface_new ();
+	interface = glade_parser_interface_new ();
 
 	if ((required = glade_project_required_libs (project)) != NULL)
 	{
 		strv = g_malloc0 (g_list_length (required) * sizeof (char *));
 		for (i = 0, list = required; list; i++, list = list->next)
-			strv[i] = g_strdup (list->data);
+			strv[i] = list->data; /* list->data is allocated for us */
 
 		interface->n_requires = g_list_length (required);
 		interface->requires   = strv;
@@ -1382,6 +1431,9 @@ glade_project_write (GladeProject *project)
 
 	g_list_free (tops);
 
+	glade_project_update_comment (project);
+	interface->comment = g_strdup (project->comment);
+	
 	return interface;
 }
 
@@ -1435,6 +1487,10 @@ glade_project_new_from_interface (GladeInterface *interface, const gchar *path)
 	project->objects = NULL;
 	project->loading = TRUE;
 
+	/* keep a comment */
+	if (interface->comment)
+		project->comment = g_strdup (interface->comment);
+	
 	for (i = 0; i < interface->n_toplevels; ++i)
 	{
 		widget = glade_widget_read ((gpointer)project, interface->toplevels[i]);
@@ -1446,15 +1502,15 @@ glade_project_new_from_interface (GladeInterface *interface, const gchar *path)
 		glade_project_add_object (project, NULL, widget->object);
 	}
 
-	/* Emit "parse-finished" signal */
-	g_signal_emit (project, glade_project_signals [PARSE_FINISHED], 0);
-	
 	/* Reset project status here too so that you get a clean
 	 * slate after calling glade_project_open().
 	 */
 	project->changed = FALSE;
 	project->loading = FALSE;
 
+	/* Emit "parse-finished" signal */
+	g_signal_emit (project, glade_project_signals [PARSE_FINISHED], 0);
+	
 	return project;	
 }
 
@@ -1515,7 +1571,7 @@ glade_project_open (const gchar *path)
 	g_return_val_if_fail (path != NULL, NULL);
 
 	if ((interface = 
-	     glade_parser_parse_file (path, NULL)) != NULL)
+	     glade_parser_interface_new_from_file (path, NULL)) != NULL)
 	{
 		if ((project = 
 		     glade_project_new_from_interface (interface, 
@@ -1527,7 +1583,7 @@ glade_project_open (const gchar *path)
 			glade_project_fix_object_props (project);
 		}
 		
-		glade_interface_destroy (interface);
+		glade_parser_interface_destroy (interface);
 	
 	}
 
@@ -1598,8 +1654,8 @@ glade_project_save (GladeProject *project, const gchar *path, GError **error)
 		return FALSE;
 	}
 
-	ret = glade_interface_dump_full (interface, path, error);
-	glade_interface_destroy (interface);
+	ret = glade_parser_interface_dump (interface, path, error);
+	glade_parser_interface_destroy (interface);
 
 	canonical_path = glade_util_canonical_path (path);
 	g_assert (canonical_path);
