@@ -38,18 +38,23 @@
 #include "glade-debug.h"
 #include "glade-placeholder.h"
 #include "glade-widget.h"
-#include "glade-widget-class.h"
+#include "glade-widget-adaptor.h"
 #include "glade-property.h"
 #include "glade-property-class.h"
 #include "glade-clipboard.h"
+#include "glade-fixed.h"
 
 #define GLADE_UTIL_SELECTION_NODE_SIZE 7
 #define GLADE_UTIL_COPY_BUFFSIZE       1024
 
+#define GLADE_DEVHELP_ICON_NAME           "devhelp"
+#define GLADE_DEVHELP_FALLBACK_ICON_FILE  "devhelp.png"
+
 /* List of widgets that have selection
  */
 static GList *glade_util_selection = NULL;
 
+static GtkTooltips *glade_tooltips = NULL;
 
 /**
  * glade_util_widget_set_tooltip:
@@ -61,17 +66,19 @@ static GList *glade_util_selection = NULL;
 void
 glade_util_widget_set_tooltip (GtkWidget *widget, const gchar *str)
 {
-	GtkTooltips *tooltips;
+	g_return_if_fail (widget != NULL);
+	if (str == NULL)
+		return;
 
-	if (GTK_WIDGET_NO_WINDOW (widget)) return;
+	if (glade_tooltips == NULL)
+	{
+		glade_tooltips = gtk_tooltips_new ();
 	
-	tooltips = gtk_tooltips_new ();
-	g_object_ref (G_OBJECT (tooltips));
-	gtk_object_sink (GTK_OBJECT (tooltips));
-	g_object_set_data_full (G_OBJECT (widget),
-				"tooltips", tooltips,
-				(GDestroyNotify) g_object_unref);
-	gtk_tooltips_set_tip (tooltips, widget, str, NULL);
+		g_object_ref (G_OBJECT (glade_tooltips));
+		gtk_object_sink (GTK_OBJECT (glade_tooltips));	
+	}
+
+	gtk_tooltips_set_tip (glade_tooltips, widget, str, NULL);
 }
 
 /**
@@ -118,11 +125,12 @@ glade_util_compose_get_type_func (const gchar *name)
 
 /**
  * glade_util_get_type_from_name:
- * @name:
+ * @name: the name of the #GType - like 'GtkWidget'.
  *
- * TODO: write me
+ * Looks up the type registering function from a plugin
+ * and initializes & returns the type for @name.
  *
- * Returns:
+ * Returns: the new #GType
  */
 GType
 glade_util_get_type_from_name (const gchar *name)
@@ -568,7 +576,11 @@ glade_util_replace (gchar *str, gchar a, gchar b)
 gchar *
 glade_util_read_prop_name (const gchar *str)
 {
-	gchar *id = g_strdup (str);
+	gchar *id;
+
+	g_return_val_if_fail (str != NULL, NULL);
+
+	id = g_strdup (str);
 
 	glade_util_replace (id, '_', '-');
 
@@ -938,22 +950,6 @@ glade_util_get_selection ()
 	return glade_util_selection;
 }
 
-/**
- * glade_util_selected_placeholder:
- *
- * Returns: The selected placeholder, if there
- * is only one selected item and that item is a placeholder.
- */
-GladePlaceholder *
-glade_util_selected_placeholder ()
-{
-	if (g_list_length (glade_util_selection) == 1 &&
-	    GLADE_IS_PLACEHOLDER (glade_util_selection->data))
-		return glade_util_selection->data;
-
-	return NULL;
-}
-
 /*
  * taken from gtk... maybe someday we can convince them to
  * expose gtk_container_get_all_children
@@ -996,75 +992,6 @@ glade_util_container_get_all_children (GtkContainer *container)
 	return g_list_reverse (children);
 }
 
-/**
- * glade_util_uri_list_parse:
- * @uri_list: the text/urilist, must be NULL terminated.
- *
- * Extracts a list of file names from a standard text/uri-list,
- * such as one you would get on a drop operation.
- * This is mostly stolen from gnome-vfs-uri.c.
- *
- * Returns: a #GList of gchars.
- */
-GList *
-glade_util_uri_list_parse (const gchar *uri_list)
-{
-	const gchar *p, *q;
-	GList *result = NULL;
-
-	g_return_val_if_fail (uri_list != NULL, NULL);
-
-	p = uri_list;
-
-	/* We don't actually try to validate the URI according to RFC
-	 * 2396, or even check for allowed characters - we just ignore
-	 * comments and trim whitespace off the ends.  We also
-	 * allow LF delimination as well as the specified CRLF.
-	 */
-	while (p)
-	{
-		if (*p != '#')
-		{
-			gchar *retval;
-			gchar *path;
-
-			while (g_ascii_isspace (*p))
-				p++;
-
-			q = p;
-			while ((*q != '\0') && (*q != '\n') && (*q != '\r'))
-				q++;
-
-			if (q > p)
-			{
-				q--;
-				while (q > p && g_ascii_isspace (*q))
-					q--;
-
-				retval = g_new (gchar, q - p + 2);
-				memcpy (retval, p, q - p + 1);
-				retval[q - p + 1] = '\0';
-
-				path = g_filename_from_uri (retval, NULL, NULL);
-				if (!path)
-				{
-					g_free (retval);
-					continue;
-				}
-
-				result = g_list_prepend (result, path);
-
-				g_free (retval);
-			}
-		}
-		p = strchr (p, '\n');
-		if (p)
-			p++;
-	}
-
-	return g_list_reverse (result);
-}
-
 /**
  * glade_util_gtkcontainer_relation:
  * @widget: a GladeWidget
@@ -1077,13 +1004,11 @@ glade_util_uri_list_parse (const gchar *uri_list)
 gboolean
 glade_util_gtkcontainer_relation (GladeWidget *parent, GladeWidget *widget)
 {
-	GladeSupportedChild *support;
 	g_return_val_if_fail (GLADE_IS_WIDGET (parent), FALSE);
 	g_return_val_if_fail (GLADE_IS_WIDGET (widget), FALSE);
-	return (GTK_IS_CONTAINER (parent->object)                      &&
-		(support = glade_widget_class_get_child_support
-		 (parent->widget_class, widget->widget_class->type))   &&
-		(support->type == GTK_TYPE_WIDGET));
+
+	return (GTK_IS_CONTAINER (parent->object) &&
+		GTK_IS_WIDGET (widget->object));
 }
 
 /**
@@ -1118,9 +1043,9 @@ glade_util_widget_pastable (GladeWidget *child,
 {
 	g_return_val_if_fail (GLADE_IS_WIDGET (child),  FALSE);
 	g_return_val_if_fail (GLADE_IS_WIDGET (parent), FALSE);
-	return (glade_widget_class_get_child_support
-		(parent->widget_class,
-		 child->widget_class->type) != NULL) ? TRUE : FALSE;
+
+	/* FIXME: what to do now ? */
+	return TRUE;
 }
 
 /**
@@ -1136,8 +1061,8 @@ glade_util_count_placeholders (GladeWidget *parent)
 	GList *list, *children;
 
 	/* count placeholders */
-	if ((children = glade_widget_class_container_get_children
-	     (parent->widget_class, parent->object)) != NULL)
+	if ((children = glade_widget_adaptor_get_children
+	     (parent->adaptor, parent->object)) != NULL)
 	{
 		for (list = children; list && list->data; list = list->next)
 		{
@@ -1146,6 +1071,7 @@ glade_util_count_placeholders (GladeWidget *parent)
 		}
 		g_list_free (children);
 	}
+
 	return placeholders;
 }
 
@@ -1408,7 +1334,7 @@ glade_util_copy_file (const gchar  *src_path,
 	if (glade_util_canonical_match (src_path, dest_path))
 		return FALSE;
 
-	if (g_file_test (dest_path, G_FILE_TEST_IS_REGULAR) == TRUE)
+	if (g_file_test (dest_path, G_FILE_TEST_IS_REGULAR) != FALSE)
 		if (glade_util_ui_message
 		    (glade_app_get_window(), GLADE_UI_YES_OR_NO,
 		     _("%s exists.\nDo you want to replace it?"), dest_path) == FALSE)
@@ -1553,33 +1479,15 @@ glade_util_load_library (const gchar *library_name)
 	GModule *module;
 
 	path = g_module_build_path (glade_modules_dir, library_name);
-	if (!path)
+
+	if ((module = g_module_open (path, G_MODULE_BIND_LAZY)) == NULL)
 	{
-		g_warning (_("Not enough memory."));
-		return NULL;
+		g_warning (_("Unable to open the module %s (%s)."),
+			   path, g_module_error());
 	}
 
-	module = g_module_open (path, G_MODULE_BIND_LAZY);
 	g_free (path);
 
-	if (!module)
-	{
-		path = g_module_build_path (NULL, library_name);
-		if (!path)
-		{
-			g_warning (_("Not enough memory."));
-			return NULL;
-		}
-
-		module = g_module_open (path, G_MODULE_BIND_LAZY);
-		g_free (path);
-	}
-
-	if (!module)
-	{
-		g_warning (_("Unable to open the module %s."), library_name);
-	}
-
 	return module;
 }
 
@@ -1605,3 +1513,244 @@ glade_util_file_is_writeable (const gchar *path)
 	}
 	return FALSE;
 }
+
+/**
+ * glade_util_have_devhelp:
+ *
+ * Returns: whether the devhelp module is loaded
+ */
+gboolean
+glade_util_have_devhelp (void)
+{
+	static gint  have_devhelp = -1;
+	gchar       *ptr;
+	gint         cnt, ret, major, minor;
+	GError      *error = NULL;
+
+#define DEVHELP_OLD_MESSAGE  \
+    "The DevHelp installed on your system is too old, " \
+    "devhelp feature will be disabled."
+
+#define DEVHELP_MISSING_MESSAGE  \
+    "No DevHelp installed on your system, " \
+    "devhelp feature will be disabled."
+
+	if (have_devhelp < 0)
+	{
+		have_devhelp = 0;
+
+		if ((ptr = g_find_program_in_path ("devhelp")) != NULL)
+		{
+			g_free (ptr);
+
+
+			if (g_spawn_command_line_sync ("devhelp --version", 
+						       &ptr, NULL, &ret, &error))
+			{
+				/* If we have a successfull return code.. parse the output.
+				 */
+				if (ret == 0)
+				{
+					
+					if ((cnt = sscanf (ptr, "DevHelp %d.%d\n", 
+							   &major, &minor)) == 2)
+					{
+						/* Devhelp 0.12 required.
+						 */
+						if (major >= 0 &&
+						    minor >= 12)
+							have_devhelp = 1;
+						else	
+							g_message (DEVHELP_OLD_MESSAGE);
+				} else if (ptr != NULL || 
+						   strlen (ptr) > 0) 
+						g_warning ("devhelp had unparsable output: "
+							   "'%s' (parsed %d elements)", ptr, cnt);
+					else
+						g_message (DEVHELP_OLD_MESSAGE);
+				} else {
+					g_warning ("devhelp had bad return code: '%d'", ret);
+				}
+			} else {
+				g_warning ("Error trying to launch devhelp: %s",
+					   error->message);
+				g_error_free (error);
+			}
+		} else
+			g_message (DEVHELP_MISSING_MESSAGE);
+		
+	}
+	return have_devhelp;
+}
+
+/**
+ * glade_util_get_devhelp_icon:
+ * @size: the preferred icon size
+ *
+ * Creates an image displaying the devhelp icon.
+ *
+ * Returns: a #GtkImage
+ */
+GtkWidget*
+glade_util_get_devhelp_icon (GtkIconSize size)
+{
+	GtkIconTheme *icon_theme;
+	GdkScreen *screen;
+	GtkWidget *image;
+	gchar *path;
+
+	image = gtk_image_new ();
+	screen = gtk_widget_get_screen (GTK_WIDGET (image));
+	icon_theme = gtk_icon_theme_get_for_screen (screen);
+
+	if (gtk_icon_theme_has_icon (icon_theme, GLADE_DEVHELP_ICON_NAME))
+	{
+		gtk_image_set_from_icon_name (GTK_IMAGE (image), GLADE_DEVHELP_ICON_NAME, size);
+	}
+	else
+	{
+		path = g_build_filename (glade_pixmaps_dir, GLADE_DEVHELP_FALLBACK_ICON_FILE, NULL);
+
+		gtk_image_set_from_file (GTK_IMAGE (image), path);
+
+		g_free (path);
+	}
+
+	return image;
+}
+
+/**
+ * glade_util_search_devhep:
+ * @devhelp: the devhelp widget created by the devhelp module.
+ * @book: the devhelp book (or %NULL)
+ * @page: the page in the book (or %NULL)
+ * @search: the search string (or %NULL)
+ *
+ * Envokes devhelp with the appropriate search string
+ *
+ */
+void
+glade_util_search_devhelp (const gchar *book,
+			   const gchar *page,
+			   const gchar *search)
+{
+	GError *error = NULL;
+	gchar  *book_comm = NULL, *page_comm = NULL;
+	gchar  *string;
+
+	g_return_if_fail (glade_util_have_devhelp ());
+
+	if (book) book_comm = g_strdup_printf ("book:%s ", book);
+	if (page) page_comm = g_strdup_printf ("page:%s ", page);
+
+	string = g_strdup_printf ("devhelp -s \"%s%s%s\"", 
+				   book_comm ? book_comm : "",
+				   page_comm ? page_comm : "",
+				   search ? search : "");
+
+	if (g_spawn_command_line_async (string, &error) == FALSE)
+	{
+		g_warning ("Error envoking devhelp: %s", error->message);
+		g_error_free (error);
+	}
+
+	g_free (string);
+	if (book_comm) g_free (book_comm);
+	if (page_comm) g_free (page_comm);
+}
+
+static GladeWidget *grabed_widget = NULL;
+void
+glade_util_set_grabed_widget (GladeWidget *gwidget)
+{
+	grabed_widget = gwidget;
+}
+
+gboolean
+glade_util_deep_fixed_event (GtkWidget   *widget,
+			     GdkEvent    *event,
+			     GladeWidget *gwidget)
+{
+	GladeWidget *event_widget, *search;
+
+	if (grabed_widget)
+		event_widget = grabed_widget;
+	else
+		event_widget = glade_widget_event_widget ();
+
+	/* Look for a child GladeFixed
+	 */
+	for (search = event_widget;
+	     search && search != gwidget && GLADE_IS_FIXED (search) == FALSE;
+	     search = search->parent);
+
+	if (search && GLADE_IS_FIXED (search) && search != gwidget)
+		return GLADE_WIDGET_GET_CLASS (search)->event (widget, event, search);
+
+	return FALSE;
+}
+
+GtkWidget *
+glade_util_get_placeholder_from_pointer (GtkContainer *container)
+{
+	GtkWidget *toplevel;
+	GtkWidget *retval = NULL, *child;
+	GList *c, *l;
+	gint x, y, x2, y2;
+
+	g_return_val_if_fail (GTK_IS_CONTAINER (container), NULL);
+
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (container));
+
+	gtk_widget_get_pointer (toplevel, &x, &y);
+	
+	for (c = l = glade_util_container_get_all_children (container);
+	     l;
+	     l = g_list_next (l))
+	{
+		child = l->data;
+		
+		if (GLADE_IS_PLACEHOLDER (child) &&
+		    GTK_WIDGET_MAPPED (child))
+		{
+			gtk_widget_translate_coordinates (toplevel, child,
+							  x, y, &x2, &y2);
+
+			
+			if (x2 >= 0 && y2 >= 0 &&
+			    x2 <= child->allocation.width &&
+			    y2 <= child->allocation.height)
+			{
+				retval = child;
+				break;
+			}
+		}
+	}
+	
+	g_list_free (c);
+
+	return retval;
+}
+
+/**
+ * glade_util_object_is_loading:
+ * @object: A #GObject
+ *
+ * Returns: Whether the object's project is being loaded or not.
+ *       
+ */
+gboolean
+glade_util_object_is_loading (GObject *object)
+{
+	GladeProject *project;
+	GladeWidget *widget;
+
+	g_return_val_if_fail (G_IS_OBJECT (object), FALSE);
+	
+	widget = glade_widget_get_from_gobject (object);
+	g_return_val_if_fail (GLADE_IS_WIDGET (widget), FALSE);
+	
+	project = glade_widget_get_project (widget);
+	
+	return glade_project_is_loading (project);
+}
